import greenfoot.*;

/**
 * Class Seizure: a world at which you try to avoid staring.
 * 
 * WARNING: May cause episodes of epiletic fits.  Those with epilepsy are at their own risk.
 */
public class Seizure extends World
{
    int[] xs = { 10, 31, 41, 31, 10, 0 }, ys = { 0, 0, 16, 32, 32, 16 };
    Color[] colors =
        { Color.BLACK, Color.BLUE, Color.CYAN, Color.PINK, Color.RED, Color.ORANGE, Color.YELLOW, Color.GREEN };
    
    GreenfootImage bg, hex = new GreenfootImage(42, 33);
    int[][] hexColors = new int[getWidth()/2][getHeight()];
    int shift;
    
    Actor btnSeizure, btnExpandingText, mouseOn;

    /**
     * Seizure constructor: sets field values, initializes background image and adds actors
     */
    public Seizure()
    {
        super(30, 30, 16);
        bg = getBackground();
        fillArray();
        drawPolygons();
        addObject(new ExpandingText("DO NOT\nSTARE", 140, Color.BLACK, 8), 15, 15);
        
        addObject(btnSeizure = getNewButton("Seizure"), 9, 28);
        addObject(btnExpandingText = getNewButton("ExpandingText"), 21, 28);
    }
    
    /**
     * Method act: runs the changing of the background colors and handles button clicks
     */
    public void act()
    {
        shift = (shift+1)%8;
        drawPolygons();
        
        if (!Greenfoot.mouseClicked(null)) return;
        if (Greenfoot.mouseClicked(btnSeizure)) Greenfoot.setWorld(new TextFileViewer("Seizure.txt", this));
        if (Greenfoot.mouseClicked(btnExpandingText)) Greenfoot.setWorld(new TextFileViewer("ExpandingText.txt", this));
    }
    
    /**
     * Method fillArray: sets color index values for all hexagons; for elements not associated with a hexagon,
     * the value is set to '-1'
     */
    private void fillArray()
    {
        for (int i=0; i<getWidth()/2; i++) for (int j=0; j<getHeight(); j++)
            hexColors[i][j] = (i+j)%2 == 0 ? Greenfoot.getRandomNumber(8) : -1;
    }
    
    /**
     * Method drawPolygons: draws/redraws all hexagons onto background image
     */
    private void drawPolygons()
    {
        for (int i=0; i<getWidth()/2; i++) for (int j=0; j<getHeight(); j++)
        {
            if (hexColors[i][j] == -1) continue;
            hex.clear();
            hex.setColor(colors[(hexColors[i][j]+shift)%8]);
            hex.fillPolygon(xs, ys, 6);
            hex.setColor(Color.WHITE);
            hex.drawPolygon(xs, ys, 6);
            bg.drawImage(hex, i*32-4, j*16-8);
        }
    }

    /**
     * Method getNewButton: creates an Actor object with a button image with the caption given; the
     * image of the button is given the ability to change intensity as the mouse moves on and off it.
     *
     * @param caption the text to display on the button
     * @return the button Actor object created
     */
    private Actor getNewButton(String caption)
    {
        // create the image for the actor not yet created
        GreenfootImage base = new GreenfootImage(140, 30); // create the base image for the button
        base.fill(); // fill with black (default drawing color) to be used for the frame
        base.setColor(new Color(192, 192, 255)); // set drawing color to a light blue
        base.fillRect(3, 3, 134, 24); // fill background of button leaving the outer frame
        GreenfootImage text = new GreenfootImage(caption, 24, Color.BLACK, new Color(0, 0, 0, 0)); // create text image
        base.drawImage(text, 70-text.getWidth()/2, 15-text.getHeight()/2); // draw text image onto base image
        base.setTransparency(128); // adjust the intensity of the image to 'mouse not over' state
        Actor button = new Actor() // create the Actor
        {
            /**
             * Method act (for button Actor): changes intensity of image for mouse hovering action
             */
            public void act()
            {
                // gaining mouse hover
                if (mouseOn == null && Greenfoot.mouseMoved(this))
                {
                    mouseOn = this;
                    getImage().setTransparency(255);
                }
                // losing mouse hover
                if (mouseOn == this && Greenfoot.mouseMoved(null) && !Greenfoot.mouseMoved(this))
                {
                    mouseOn = null;
                    getImage().setTransparency(128);
                }
            }
        };
        button.setImage(base); // give button its image
        return button; // return the button Actor object
    }
}